﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Web.Script.Serialization;
using MongoDB.Driver;

namespace MongoDbConsole
{
    public static class ExtensionMethods
    {
        public static IEnumerable<T> ToIEnumerable<T>(this IEnumerable<Document> documents)
        {
            var list = new List<T>(); 

            foreach(var document in documents)
            {
                list.Add(document.ToClass<T>());
            }

            return list; 
        }

        public static T ToClass<T>(this Document source)
        {
            if(source == null) throw new ArgumentNullException("Document is null!");

            return new JavaScriptSerializer().Deserialize<T>(source.ToString()); 
        }

        public static Document ToDocument(this object source)
        {
            var document = SerializeMember(source) as Document;
            return document; 
        } 

        private static object SerializeMember(object source)
        {
            // get the properties 

            if(!Type.GetTypeCode(source.GetType()).Equals(TypeCode.Object))
                return source;

            // if the object is IEnumerable 

            var enumerable = source as IEnumerable; 

            if(enumerable != null)
            {
                var documents = new List<Object>(); 

                foreach(var doc in enumerable)
                {
                    documents.Add(SerializeMember(doc)); 
                }

                return documents.ToArray(); 
            }
           
                var properties = source.GetType().GetProperties(BindingFlags.Instance | BindingFlags.Public);

                var document = new Document();

                foreach (var property in properties)
                {
                    var propertyValue = property.GetValue(source, null);

                    if (propertyValue == null) continue;

                    document.Add(property.Name, SerializeMember(propertyValue));
                }

                return document;
        }
    }
}
